@@ -19,9 +19,9 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py,v 1.48 2006/10/12 18:13:02 wwalker Exp $"
-__version__   = "$Revision: 1.48 $"
-__date__      = "$Date: 2006/10/12 18:13:02 $"
+__id__        = "$Id: atspi.py,v 1.50 2006/10/26 16:58:17 richb Exp $"
+__version__   = "$Revision: 1.50 $"
+__date__      = "$Date: 2006/10/26 16:58:17 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -163,7 +163,7 @@ class Registry:
             debug.println(debug.LEVEL_CONFIGURATION,
                           "atspi.start: using bonobo.main; "
                           + "gilSleepTime=%f" % settings.gilSleepTime)
-            if settings.gilSleepTime:
+            if settings.useBlockPreventor and settings.gilSleepTime:
                 gobject.idle_add(self.__blockPreventor)
             bonobo.main()
         else:
@@ -1027,9 +1027,6 @@ class Accessible:
             except:
                 pass
 
-        if settings.cacheValues:
-            self.relations = relations
-
         return relations
 
     def __get_app(self):
