@@ -25,17 +25,32 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py 1245 2006-07-28 11:33:32Z wwalker $"
-__version__   = "$Revision: 1245 $"
-__date__      = "$Date: 2006-07-28 07:33:32 -0400 (Fri, 28 Jul 2006) $"
+__id__        = "$Id: braille.py,v 1.66 2006/11/06 16:41:03 wwalker Exp $"
+__version__   = "$Revision: 1.66 $"
+__date__      = "$Date: 2006/11/06 16:41:03 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import logging
+log = logging.getLogger("braille")
+
 import signal
 import threading
 
 import atspi
-import brl
+
+# We'll use the official BrlAPI pythons (as of BrlTTY 3.8) if they
+# are available.  Otherwise, we'll fall back to our own bindings.
+#
+try:
+    import brlapi
+    brlAPI = None
+    useBrlAPIBindings = True
+    brlAPIRunning = False
+except:
+    import brl
+    useBrlAPIBindings = False
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -138,6 +153,18 @@ CMD_MAX               = 0x44
 BRL_FLG_REPEAT_INITIAL= 0x800000
 BRL_FLG_REPEAT_DELAY  = 0x400000
 
+# Common names for most used BrlTTY commands, to be shown in the GUI:
+# ATM, the ones used in default.py are:
+#
+command_name = {}
+command_name[CMD_FWINLT]   = _("Line Left")
+command_name[CMD_FWINRT]   = _("Line Right")
+command_name[CMD_LNUP]     = _("Line Up")
+command_name[CMD_LNDN]     = _("Line Down")
+command_name[CMD_TOP_LEFT] = _("Top Left")
+command_name[CMD_BOT_LEFT] = _("Bottom Right")
+command_name[CMD_HOME]     = _("Cursor Position")
+
 # The size of the physical display (width, height).  The coordinate system of
 # the display is set such that the upper left is (0,0), x values increase from
 # left to right, and y values increase from top to bottom.
@@ -210,10 +237,11 @@ class Region:
         if not string:
             string = ""
 
+        string = string.decode("UTF-8")
         if string[-1:] == "\n":
             string = string[:-1]
+        self.string = string.encode("UTF-8")
 
-        self.string = string
         self.cursorOffset = cursorOffset
 
     def processCursorKey(self, offset):
@@ -304,7 +332,7 @@ class Text(Region):
         self.label = label
         if self.label:
             string = self.label + " " + string
-            cursorOffset += len(self.label) + 1
+            cursorOffset += len(self.label.decode("UTF-8")) + 1
 
         Region.__init__(self, string, cursorOffset)
 
@@ -315,12 +343,11 @@ class Text(Region):
         True.  Otherwise, return False.
         """
 
-        result = util.getTextLineAtCaret(self.accessible)
-        caretOffset = result[1]
-        lineOffset = result[2]
+        [string, caretOffset, lineOffset] = \
+                 util.getTextLineAtCaret(self.accessible)
         cursorOffset = caretOffset - lineOffset
         if self.label:
-            cursorOffset += len(self.label) + 1
+            cursorOffset += len(self.label.decode("UTF-8")) + 1
 
         if lineOffset != self.lineOffset:
             return False
@@ -338,7 +365,7 @@ class Text(Region):
         scrolled off the display."""
 
         if self.label:
-            offset = offset - len(self.label) - 1
+            offset = offset - len(self.label.decode("UTF-8")) - 1
             if offset < 0:
                 return
 
@@ -423,9 +450,9 @@ class Line:
         focusOffset = -1
         for region in self.regions:
             if region == _regionWithFocus:
-                focusOffset = len(string)
+                focusOffset = len(string.decode("UTF-8"))
             if region.string:
-                # [[[TODO: WDW - HACK: Replace unicode ellipses with "..."
+                # [[[TODO: WDW - HACK: Replace UTF-8 ellipses with "..."
                 # The ultimate solution is to get i18n support into
                 # BrlTTY.]]]
                 #
@@ -450,10 +477,10 @@ class Line:
         pos = 0
         for region in self.regions:
             string = string + region.string
-            if len(string) > offset:
+            if len(string.decode("UTF-8")) > offset:
                 break
             else:
-                pos = len(string)
+                pos = len(string.decode("UTF-8"))
 
         return [region, offset - pos]
 
@@ -590,7 +617,11 @@ def refresh(panToCursor=True, targetCursorCell=0):
     global monitor
 
     if len(_lines) == 0:
-        brl.writeText(0, "")
+        if useBrlAPIBindings:
+            if brlAPIRunning:
+                brlAPI.writeText("", 0)
+        else:
+            brl.writeText(0, "")
         return
 
     # Now determine the location of the cursor.  First, we'll figure
@@ -616,7 +647,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
     # right of the display if we need to pan right.
     #
     if panToCursor and (cursorOffset >= 0):
-        if len(string) <= _displaySize[0]:
+        if len(string.decode("UTF-8")) <= _displaySize[0]:
             _viewport[0] = 0
         elif targetCursorCell:
             _viewport[0] = max(0, cursorOffset - targetCursorCell + 1)
@@ -638,17 +669,36 @@ def refresh(panToCursor=True, targetCursorCell=0):
         cursorCell += 1 # Normalize to 1-based offset
 
     debug.println(debug.LEVEL_INFO, "BRAILLE LINE:  '%s'" % string)
+    log.info("line:'%s'" % string)
 
     debug.println(debug.LEVEL_INFO, "     VISIBLE:  '%s', cursor=%d" \
                   % (string[startPos:endPos], cursorCell))
+    log.info("visible:'%s'" % string[startPos:endPos])
+    log.info("cursor:%d" % cursorCell)
 
-    brl.writeText(cursorCell, string[startPos:endPos])
+    string = string.decode("UTF-8")
+    substring = string[startPos:endPos].encode("UTF-8")
+    if useBrlAPIBindings:
+        if brlAPIRunning:
+            writeStruct = brlapi.Write()
+            writeStruct.text = substring
+            writeStruct.regionBegin = 1
+            writeStruct.regionSize = len(substring.decode("UTF-8"))
+            while writeStruct.regionSize < _displaySize[0]:
+                writeStruct.text += " "
+                writeStruct.regionSize += 1
+            writeStruct.cursor = cursorCell
+            writeStruct.charset = "UTF-8"
+            brlAPI.write(writeStruct)
+            #brlAPI.writeText(substring, cursorCell)
+    else:
+        brl.writeText(cursorCell, substring)
 
     if settings.enableBrailleMonitor:
         if not monitor:
             monitor = brlmon.BrlMon(_displaySize[0])
             monitor.show_all()
-        monitor.writeText(cursorCell, string[startPos:endPos])
+        monitor.writeText(cursorCell, substring)
     elif monitor:
         monitor.destroy()
 
@@ -733,7 +783,7 @@ def panRight(panAmount=0):
         lineNum = _viewport[1]
         newX = _viewport[0] + panAmount
         [string, focusOffset] = _lines[lineNum].getLineInfo()
-        if newX < len(string):
+        if newX < len(string.decode("UTF-8")):
             _viewport[0] = newX
 
     return oldX != _viewport[0]
@@ -820,6 +870,29 @@ def _processBrailleEvent(command):
 
     return consumed
 
+def _brlAPIKeyReader():
+        """Method to read a key from the BrlAPI bindings.  This is a
+        gidle handler.
+        """
+        key = brlAPI.readKey(False)
+        if key:
+            flags = key >> 32
+            lower = key & 0xFFFFFFFF
+            keyType = lower >> 29
+            keyCode = lower & 0x1FFFFFFF
+
+            # [[TODO: WDW - HACK If we have a cursor routing key, map
+            # it back to the code we used to get with earlier versions
+            # of BrlAPI (i.e., bit 0x100 was the indicator of a cursor
+            # routing key instead of 0x1000).  This may change before
+            # the offical BrlAPI Python bindings are released.]]]
+            #
+            if keyCode & 0x10000:
+                keyCode = 0x100 | (keyCode & 0xFF)
+            if keyCode:
+                _processBrailleEvent(keyCode)
+        return brlAPIRunning
+
 def init(callback=None, tty=7):
     """Initializes the braille module, connecting to the BrlTTY driver.
 
@@ -839,20 +912,38 @@ def init(callback=None, tty=7):
 
     _callback = callback
 
-    if brl.init(tty):
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Braille module has been initialized.")
-        brl.registerCallback(_processBrailleEvent)
+    if useBrlAPIBindings:
+        try:
+            import gobject
+            gobject.threads_init()
+            global brlAPI
+            global brlAPIRunning
+            brlAPI = brlapi.Bridge()
+            brlAPI.getTty(tty)
+            brlAPIRunning = True
+            gobject.idle_add(_brlAPIKeyReader)
+        except:
+            debug.printException(debug.LEVEL_FINEST)
+            return False
     else:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Braille module has NOT been initialized.")
-        return False
+        if brl.init(tty):
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "Braille module has been initialized.")
+            brl.registerCallback(_processBrailleEvent)
+        else:
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "Braille module has NOT been initialized.")
+            return False
 
     # [[[TODO: WDW - For some reason, BrlTTY wants to say the height of the
     # Vario is 40 so we hardcode it to 1 for now.]]]
     #
     #_displaySize = (brl.getDisplayWidth(), brl.getDisplayHeight())
-    _displaySize = [brl.getDisplayWidth(), 1]
+    if useBrlAPIBindings:
+        (x, y) = brlAPI.displaysize
+        _displaySize = [x, 1]
+    else:
+        _displaySize = [brl.getDisplayWidth(), 1]
 
     debug.println(debug.LEVEL_CONFIGURATION,
                   "braille display size = (%d, %d)" \
@@ -875,7 +966,13 @@ def shutdown():
     if not _initialized:
         return False
 
-    brl.shutdown()
+    global brlAPIRunning
+    if useBrlAPIBindings:
+        if brlAPIRunning:
+            brlAPIRunning = False
+            brlAPI.leaveTty()
+    else:
+        brl.shutdown()
 
     _initialized = False
 
