@@ -19,12 +19,21 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py,v 1.164 2006/11/02 16:07:42 richb Exp $"
-__version__   = "$Revision: 1.164 $"
-__date__      = "$Date: 2006/11/02 16:07:42 $"
+__id__        = "$Id: orca.py,v 1.171 2006/11/30 16:38:58 richb Exp $"
+__version__   = "$Revision: 1.171 $"
+__date__      = "$Date: 2006/11/30 16:38:58 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+# We're going to force the name of the app to "orca" so pygtk
+# will end up showing us as "orca" to the AT-SPI.  If we don't
+# do this, the name can end up being "-c".  See bug 364452 at
+# http://bugzilla.gnome.org/show_bug.cgi?id=364452 for more
+# information.
+#
+import sys
+sys.argv[0] = "orca"
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -39,7 +48,6 @@ import getopt
 import os
 import signal
 import string
-import sys
 import time
 
 import atspi
@@ -69,6 +77,10 @@ from orca_i18n import _           # for gettext support
 #
 _userSettings = None
 
+# Command line options that override any other settings.
+#
+_commandLineSettings = {}
+
 ########################################################################
 #                                                                      #
 # METHODS FOR HANDLING PRESENTATION MANAGERS                           #
@@ -472,7 +484,8 @@ def _isModifierKey(event_string):
     #    event.]]]
 
     modifierKeys = [ 'Alt_L', 'Alt_R', 'Control_L', 'Control_R', \
-                     'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R', "Insert" ]
+                     'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R' ]
+    modifierKeys.extend(settings.orcaModifierKeys)
 
     reply = event_string in modifierKeys
     debug.println(debug.LEVEL_FINEST,
@@ -490,7 +503,8 @@ def _isLockingKey(event_string):
 
     lockingKeys = [ "Caps_Lock", "Num_Lock", "Scroll_Lock" ]
 
-    reply = event_string in lockingKeys
+    reply = event_string in lockingKeys \
+            and not event_string in settings.orcaModifierKeys
     debug.println(debug.LEVEL_FINEST,
                   "orca._echoLockingKey: returning: %s" % reply)
     return reply
@@ -798,6 +812,10 @@ def loadUserSettings(script=None, inputEvent=None):
     speech.shutdown()
     braille.shutdown()
     mag.shutdown()
+
+    if _currentPresentationManager >= 0:
+        _PRESENTATION_MANAGERS[_currentPresentationManager].deactivate()
+
     time.sleep(1)
 
     reloaded = False
@@ -813,6 +831,12 @@ def loadUserSettings(script=None, inputEvent=None):
         except ImportError:
             debug.printException(debug.LEVEL_FINEST)
 
+    # If any settings were added to the command line, they take
+    # precedence over everything else.
+    #
+    for key in _commandLineSettings:
+        settings.__dict__[key] = _commandLineSettings[key]
+
     if settings.enableSpeech:
         try:
             speech.init()
@@ -849,6 +873,13 @@ def loadUserSettings(script=None, inputEvent=None):
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Magnification module has NOT been initialized.")
 
+    for keyName in settings.orcaModifierKeys:
+        if keyName == "Caps_Lock":
+            os.system('xmodmap -e "clear Lock"')
+
+    if _currentPresentationManager >= 0:
+        _PRESENTATION_MANAGERS[_currentPresentationManager].activate()
+
     _showMainWindowGUI()
 
     httpserver.init()
@@ -932,6 +963,22 @@ def _showQuitGUI(script=None, inputEvent=None):
 
     return True
 
+def _showFindGUI(script=None, inputEvent=None):
+    """Displays the user interace to perform an Orca Find.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    try:
+        module = __import__(settings.findModule,
+                            globals(),
+                            locals(),
+                            [''])
+        module.showFindUI()
+    except:
+        debug.printException(debug.LEVEL_SEVERE)
+        pass
+
 # If True, this module has been initialized.
 #
 _initialized = False
@@ -1144,6 +1191,9 @@ def usage():
     print _("-s, --setup, --gui-setup     Set up user preferences")
     print _("-t, --text-setup             Set up user preferences (text version)")
     print _("-n, --no-setup               Skip set up of user preferences")
+    print _("-u, --user-prefs-dir=dirname Use alternate directory for user preferences")
+    print _("-e, --enable=[speech|braille|braille-monitor|magnifier] Force use of option")
+    print _("-d, --disable=[speech|braille|braille-monitor|magnifier] Prevent use of option")
     print _("-q, --quit                   Quits Orca (if shell script used)")
     print
     print _("If Orca has not been previously set up by the user, Orca\nwill automatically launch the preferences set up unless\nthe -n or --no-setup option is used.")
@@ -1158,6 +1208,8 @@ def main():
     an exit code of 0 means normal completion and an exit code of 50
     means Orca exited because of a hang."""
 
+    global _commandLineSettings
+
     # Method to call when we think something might be hung.
     #
     settings.timeoutCallback = timeout
@@ -1191,8 +1243,20 @@ def main():
     bypassSetup     = False
     setupRequested  = False
     showGUI         = False
+
+    # We hack a little here because the shell script to start orca can
+    # conflate all of command line arguments into one string, which is
+    # not what we want.  We detect this by seeing if the length of the
+    # argument list is 1.
+    #
+    arglist = sys.argv[1:]
+    if len(arglist) == 1:
+        arglist = arglist[0].split()
+
     try:
         # ? is for help
+        # e is for enabling a feature
+        # d is for disabling a feature
         # h is for help
         # u is for alternate user preferences location
         # s is for setup
@@ -1201,10 +1265,12 @@ def main():
         # v is for version
         #
         opts, args = getopt.getopt(
-            sys.argv[1:],
-            "?u:stnv",
+            arglist,
+            "?stnvd:e:u:",
             ["help",
-             "user-prefs-dir",
+             "user-prefs-dir=",
+             "enable=",
+             "disable=",
              "setup",
              "gui-setup",
              "text-setup",
@@ -1218,6 +1284,35 @@ def main():
                     settings.userPrefsDir = userPrefsDir
                 except:
                     debug.printException(debug.LEVEL_FINEST)
+
+            if opt in ("-e", "--enable"):
+                feature = val.strip()
+                if feature == "speech":
+                    _commandLineSettings["enableSpeech"] = True
+                elif feature == "braille":
+                    _commandLineSettings["enableBraille"] = True
+                elif feature == "braille-monitor":
+                    _commandLineSettings["enableBrailleMonitor"] = True
+                elif feature == "magnifier":
+                    _commandLineSettings["enableMagnifier"] = True
+                else:
+                    usage()
+                    os._exit(2)
+
+            if opt in ("-d", "--disable"):
+                feature = val.strip()
+                if feature == "speech":
+                    _commandLineSettings["enableSpeech"] = False
+                elif feature == "braille":
+                    _commandLineSettings["enableBraille"] = False
+                elif feature == "braille-monitor":
+                    _commandLineSettings["enableBrailleMonitor"] = False
+                elif feature == "magnifier":
+                    _commandLineSettings["enableMagnifier"] = False
+                else:
+                    usage()
+                    os._exit(2)
+
             if opt in ("-s", "--gui-setup", "--setup"):
                 setupRequested = True
                 showGUI = desktopRunning
@@ -1233,6 +1328,7 @@ def main():
                 print "Orca %s" % platform.version
                 os._exit(0)
     except:
+        debug.printException(debug.LEVEL_OFF)
         usage()
         os._exit(2)
 
