@@ -24,9 +24,9 @@
 
 """Displays a GUI for the user to set Orca preferences."""
 
-__id__        = "$Id: orca_gui_prefs.py,v 1.48 2006/07/21 19:49:03 wwalker Exp $"
-__version__   = "$Revision: 1.48 $"
-__date__      = "$Date: 2006/07/21 19:49:03 $"
+__id__        = "$Id: orca_gui_prefs.py,v 1.62 2006/12/01 17:39:10 richb Exp $"
+__version__   = "$Revision: 1.62 $"
+__date__      = "$Date: 2006/12/01 17:39:10 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -36,13 +36,19 @@ import debug
 import gettext
 import gtk
 import gtk.glade
+import gobject
+import pango   # for ellipsize property constants of CellRendererText
 import locale
 
 import acss
 import orca
+import orca_glade
 import orca_prefs
+import orca_state
 import platform
 import settings
+import default    # for the default keyBindings
+import keybindings
 import speech as speech
 import speechserver as speechserver
 
@@ -50,37 +56,10 @@ from orca_i18n import _  # for gettext support
 
 OS = None
 
-class GladeWrapper:
-    """
-    Superclass for glade based applications. Just derive from this
-    and your subclass should create methods whose names correspond to
-    the signal handlers defined in the glade file. Any other attributes
-    in your class will be safely ignored.
+(HANDLER, DESCRIP, DEFAULT_KEY, MOD_MASK1, MOD_USED1, KEY1, TEXT1, MOD_MASK2, MOD_USED2, KEY2, TEXT2, MODIF) = range(12)
 
-    This class will give you the ability to do:
-        subclass_instance.GtkWindow.method(...)
-        subclass_instance.widget_name...
-    """
 
-    def __init__(self, Filename, WindowName):
-        # Load glade file.
-        self.widgets = gtk.glade.XML(Filename, WindowName, gettext.textdomain())
-        self.GtkWindow = getattr(self, WindowName)
-
-        instance_attributes = {}
-        for attribute in dir(self.__class__):
-            instance_attributes[attribute] = getattr(self, attribute)
-        self.widgets.signal_autoconnect(instance_attributes)
-
-    def __getattr__(self, attribute):   # Called when no attribute in __dict__
-        widget = self.widgets.get_widget(attribute)
-        if widget is None:
-            raise AttributeError("Widget [" + attribute + "] not found")
-        self.__dict__[attribute] = widget   # Add reference to cache.
-
-        return widget
-
-class orcaSetupGUI(GladeWrapper):
+class orcaSetupGUI(orca_glade.GladeWrapper):
 
     def _init(self):
         """Initialize the Orca configuration GUI. Read the users current
@@ -99,6 +78,174 @@ class orcaSetupGUI(GladeWrapper):
         self.speechServersModel = self._initComboBox(self.speechServers)
         self.voicesModel        = self._initComboBox(self.voices)
 
+        # ***** Key Bindings treeview initialization *****
+
+        self.keyBindView = self.widgets.get_widget("keyBindingsTreeview")
+        self.keyBindingsModel = gtk.ListStore(
+            gobject.TYPE_STRING,  # Handler name
+            gobject.TYPE_STRING,  # Description
+            gobject.TYPE_STRING,  # Default Orca Key
+            gobject.TYPE_STRING,  # Modifier mask 1
+            gobject.TYPE_STRING,  # Used Modifiers 1
+            gobject.TYPE_STRING,  # Modifier key names 1
+            gobject.TYPE_STRING,  # User Defined Key 1
+            gobject.TYPE_STRING,  # Modifier mask 2
+            gobject.TYPE_STRING,  # Used Modifiers 2
+            gobject.TYPE_STRING,  # Modifier key names 2
+            gobject.TYPE_STRING,  # User Defined Key 2
+            gobject.TYPE_BOOLEAN) # Key Modified by User
+        self.keyBindView.set_model(self.keyBindingsModel)
+        self.keyBindView.set_headers_visible(True)
+
+        # Treeview Column Implementation:
+        #      (HANDLER,    DESCRIP,    DEFAULT_KEY,
+        #       MOD_MASK1,  MOD_USED1,  KEY1,       TEXT1,
+        #       MOD_MASK2,  MOD_USED2,  KEY2,       TEXT2,
+        #       MODIF)
+
+        # HANDLER
+        #
+        column = gtk.TreeViewColumn(_("Handler"),
+                                    gtk.CellRendererText(),
+                                    text=HANDLER)
+        column.set_resizable(True)
+        column.set_visible(False)
+        column.set_sort_column_id(HANDLER)
+        self.keyBindView.append_column(column)
+
+        # DESCRIP
+        #
+        rendererText = gtk.CellRendererText()
+        rendererText.set_property("ellipsize", pango.ELLIPSIZE_END)
+        column = gtk.TreeViewColumn(_("Function"), rendererText, text=DESCRIP)
+        column.set_resizable(True)
+        column.set_min_width(360)
+        column.set_sort_column_id(DESCRIP)
+        self.keyBindView.append_column(column)
+
+        # DEFAULT_KEY
+        #
+        column = gtk.TreeViewColumn(_("Default Key"),
+                                    gtk.CellRendererText(),
+                                    text=DEFAULT_KEY)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(DEFAULT_KEY)
+        self.keyBindView.append_column(column)
+
+        # MOD_MASK1
+        #
+        column = gtk.TreeViewColumn(_("Mod.Mask 1"),
+                                    gtk.CellRendererText(),
+                                    text=MOD_MASK1)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MOD_MASK1)
+        self.keyBindView.append_column(column)
+
+        # MOD_USED1
+        #
+        column = gtk.TreeViewColumn(_("Use Mod.1"),
+                                    gtk.CellRendererText(),
+                                    text=MOD_USED1)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MOD_USED1)
+        self.keyBindView.append_column(column)
+
+        # KEY1
+        #
+        column = gtk.TreeViewColumn(_("Key1"),
+                                    gtk.CellRendererText(),
+                                    text=KEY1)
+        column.set_resizable(True)
+        column.set_visible(False)
+        column.set_sort_column_id(KEY1)
+        self.keyBindView.append_column(column)
+
+        # TEXT1
+        #
+        rendererText = gtk.CellRendererText()
+        #rendererText.set_property('editable', True)
+        #rendererText.connect("editing-started",
+        #                     self.editingKey,
+        #                     self.keyBindingsModel)
+        #rendererText.connect('edited',
+        #                     self.editedKey,
+        #                     self.keyBindingsModel)
+        column = gtk.TreeViewColumn(_("Key Binding"),
+                                    rendererText,
+                                    text=TEXT1)
+        column.set_resizable(True)
+        column.set_sort_column_id(TEXT1)
+        self.keyBindView.append_column(column)
+
+        # MOD_MASK2
+        #
+        column = gtk.TreeViewColumn(_("Mod.Mask 2"),
+                                    gtk.CellRendererText(),
+                                    text=MOD_MASK2)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MOD_MASK2)
+        self.keyBindView.append_column(column)
+
+        # MOD_USED2
+        #
+        column = gtk.TreeViewColumn(_("Use Mod.2"),
+                                    gtk.CellRendererText(),
+                                    text=MOD_USED2)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MOD_USED2)
+        self.keyBindView.append_column(column)
+
+        # KEY2
+        #
+        column = gtk.TreeViewColumn(_("Key2"), rendererText, text=KEY2)
+        column.set_resizable(True)
+        column.set_visible(False)
+        column.set_sort_column_id(KEY2)
+        self.keyBindView.append_column(column)
+
+        # TEXT2
+        #
+        rendererText = gtk.CellRendererText()
+        #rendererText.set_property('editable', True)
+        #rendererText.connect("editing-started",
+        #                     self.editingKey,
+        #                     self.keyBindingsModel)
+        #rendererText.connect('edited',
+        #                     self.editedKey,
+        #                     self.keyBindingsModel)
+        column = gtk.TreeViewColumn(_("Alternate"),
+                                    gtk.CellRendererText(),
+                                    text=TEXT2)
+        column.set_resizable(True)
+        column.set_sort_column_id(TEXT2)
+        self.keyBindView.append_column(column)
+
+        # MODIF
+        #
+        rendererToggle = gtk.CellRendererToggle()
+        rendererToggle.set_property('activatable',False)
+        column = gtk.TreeViewColumn(_("Modified"),
+                                    rendererToggle,
+                                    active=MODIF)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MODIF)
+        self.keyBindView.append_column(column)
+
+        # Populates the treeview with all the keybindings:
+        #
+        self._populateKeyBindings()
+
+        self.keyBindView.show()
+
+        self.window = self.widgets.get_widget("orcaSetupWindow")
+        self.window.resize(730,580);
+
         self._setKeyEchoItems()
 
         factories = speech.getSpeechServerFactories()
@@ -157,7 +304,6 @@ class orcaSetupGUI(GladeWrapper):
         self.serverChoices = None
         self._setupServers()
         self._setupVoices()
-        self.prefsDict["enableSpeech"] = True
         self._initGUIState()
         self.initializing = False
 
@@ -204,8 +350,8 @@ class orcaSetupGUI(GladeWrapper):
         if serverPrefs:
             self._setServerChoice(self.serverChoices, serverPrefs[0])
 
-        voiceType = self.voiceType.get_active_text()
         self.voiceType.set_active(0)
+        voiceType = self.voiceType.get_active_text()
         self._setVoiceSettingForVoiceType(voiceType)
 
         if prefs["verbalizePunctuationStyle"] == \
@@ -233,6 +379,9 @@ class orcaSetupGUI(GladeWrapper):
         self.speechIndentationCheckbutton.set_active(\
             prefs["enableSpeechIndentation"])
 
+        self.speakBlankLinesCheckButton.set_active(\
+            prefs["speakBlankLines"])
+
         # Braille pane.
         #
         self.brailleSupportCheckbutton.set_active(prefs["enableBraille"])
@@ -387,6 +536,23 @@ class orcaSetupGUI(GladeWrapper):
         index = self._getComboBoxIndex(self.magMouseTrackingComboBox, mode)
         self.magMouseTrackingComboBox.set_active(index)
 
+        # Get the magnification source and target displays.
+        #
+        sourceDisplay = prefs["magSourceDisplay"]
+        self.magSourceDisplayEntry.set_text(sourceDisplay)
+
+        targetDisplay = prefs["magTargetDisplay"]
+        self.magTargetDisplayEntry.set_text(targetDisplay)
+
+        # General pane.
+        #
+        self.showMainWindowCheckButton.set_active(prefs["showMainWindow"])
+
+        if prefs["keyboardLayout"] == settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP:
+            self.generalDesktopButton.set_active(True)
+        else:
+            self.generalLaptopButton.set_active(True)
+
     def _getComboBoxIndex(self, combobox, str):
         """ For each of the entries in the given combo box, look for str.
             Return the index of the entry if str is found.
@@ -395,8 +561,8 @@ class orcaSetupGUI(GladeWrapper):
         - combobox: the GtkComboBox to search.
         - str: the string to search for.
 
-        Returns the index of the first entry in combobox with str, on
-        None if not found.
+        Returns the index of the first entry in combobox with str, or
+        0 if not found.
         """
 
         model = combobox.get_model()
@@ -407,7 +573,7 @@ class orcaSetupGUI(GladeWrapper):
                 return i
             iter = model.iter_next(iter)
 
-        return None
+        return 0
 
     def _setupServers(self):
         """Get the list of speech servers for the current speech factory.
@@ -717,7 +883,9 @@ class orcaSetupGUI(GladeWrapper):
         # then we can fail gracefully.
         #
         try:
-            self.orcaSetupWindow.window.set_user_time(orca.lastTimestamp)
+            self.orcaSetupWindow.realize()
+            self.orcaSetupWindow.window.set_user_time(\
+                orca_state.lastInputEventTimestamp)
         except AttributeError:
             debug.printException(debug.LEVEL_FINEST)
 
@@ -764,6 +932,75 @@ class orcaSetupGUI(GladeWrapper):
 
         speech.speak(text)
 
+    def _addAlternateKeyBinding(self, kb):
+        """Adds an alternate keybinding to the existing handler and
+        returns true.  In case it doesn't exist yet, just returns
+        false.
+
+        Argument:
+        - kb: the keybinding to be added as an alternate keybinding.
+        """
+
+        iter = self.keyBindingsModel.get_iter_first()
+        exist = False
+
+        for row in self.keyBindingsModel:
+            if row[DESCRIP]==kb.handler._description:
+                exist = True
+                row[MOD_MASK2] = kb.modifier_mask
+                row[MOD_USED2] = kb.modifiers
+                row[KEY2]      = kb.keysymstring
+                row[TEXT2]     = keybindings.getModifierNames(kb.modifiers) \
+                                 + kb.keysymstring
+
+        return exist
+
+    def _insert_row(self, kb, modif=False):
+        """Appends a new row with the new keybinding data to the treeview
+
+        Arguments:
+        - kb:    the new keybinding.
+        - modif: whether to check the modified field or not.
+
+        Returns a gtk.TreeIter pointing at the new row.
+        """
+
+        model = self.keyBindingsModel
+
+        myiter = model.insert_after(None, None)
+        model.set (myiter,
+                   HANDLER,     kb.handler,
+                   DESCRIP,     kb.handler._description,
+                   DEFAULT_KEY, kb.keysymstring,
+                   MOD_MASK1,   kb.modifier_mask,
+                   MOD_USED1,   kb.modifiers,
+                   KEY1,        kb.keysymstring,
+                   TEXT1,       keybindings.getModifierNames(kb.modifiers) \
+                                + kb.keysymstring,
+                   MODIF,       modif)
+        return myiter
+
+    def _populateKeyBindings(self):
+        """Fills the TreeView with the list of Orca keybindings
+        """
+        self.keyBindingsModel.clear()
+
+        self.kbindings = keybindings.KeyBindings()
+
+        # KeyBindings from the default script, in default.py (Orca's default)
+        #
+        self.kbindingsDef = default.Script(None).getKeyBindings()
+        for kb in self.kbindingsDef.keyBindings:
+            if not self.kbindings.hasKeyBinding(kb, typeOfSearch="strict"):
+                if not self._addAlternateKeyBinding(kb):
+                    self._insert_row(kb)
+
+        self.orcaModKeyEntry = self.widgets.get_widget("orcaModKeyEntry")
+        self.orcaModKeyEntry.set_text(
+            settings.orcaModifierKeys.__str__()[1:-1].replace("'",""))
+
+        self.keyBindingsModel.set_sort_column_id(TEXT1, gtk.SORT_ASCENDING)
+
     def speechSupportChecked(self, widget):
         """Signal handler for the "toggled" signal for the
            speechSupportCheckbutton GtkCheckButton widget. The user has
@@ -849,6 +1086,18 @@ class orcaSetupGUI(GladeWrapper):
         enable = widget.get_active()
         self.prefsDict["enableSpeechIndentation"] = enable
 
+    def speakBlankLinesChecked(self, widget):
+        """Signal handler for the "toggled" signal for the
+           speakBlankLinesCheckButton GtkCheckButton widget. The user has
+           [un]checked the 'Speak blank lines' checkbox.
+           Set the 'speakBlankLines' preference to the new value.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.prefsDict["speakBlankLines"] = widget.get_active()
+
     def brailleSupportChecked(self, widget):
         """Signal handler for the "toggled" signal for the
            brailleSupportCheckbutton GtkCheckButton widget. The user has
@@ -1350,6 +1599,70 @@ class orcaSetupGUI(GladeWrapper):
 
         self.prefsDict["enableMagZoomerColorInversion"] = widget.get_active()
 
+    def magSourceDisplayChanged(self, widget):
+        """Signal handler for the "changed" signal for the
+           magSourceDisplayDisplayEntry GtkEntry widget.
+           The user has changed the value of the magnification source
+           display. Set the 'magSourceDisplay' preference
+           to the new value.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.prefsDict["magSourceDisplay"] = widget.get_text()
+
+    def magTargetDisplayChanged(self, widget):
+        """Signal handler for the "changed" signal for the
+           magTargetDisplayEntry GtkEntry widget.
+           The user has changed the value of the magnification target
+           display. Set the 'magTargetDisplay' preference
+           to the new value.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.prefsDict["magTargetDisplay"] = widget.get_text()
+
+    def showMainWindowChecked(self, widget):
+        """Signal handler for the "toggled" signal for the
+           showMainWindowCheckButton GtkCheckButton widget.
+           The user has [un]checked the 'Show Orca main window'
+           checkbox. Set the 'showMainWindow' preference
+           to the new value.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.prefsDict["showMainWindow"] = widget.get_active()
+
+    def keyboardLayoutChanged(self, widget):
+        """Signal handler for the "toggled" signal for the generalDesktopButton,
+           or generalLaptopButton GtkRadioButton widgets. The user has
+           toggled the keyboard layout value. If this signal was
+           generated as the result of a radio button getting selected
+           (as opposed to a radio button losing the selection), set the
+           'keyboardLayout' preference to the new value. Also set the
+           matching list of Orca modifier keys
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        if widget.get_active():
+            if widget.get_label() == _("_Desktop"):
+                self.prefsDict["keyboardLayout"] = \
+                    settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP
+                self.prefsDict["orcaModifierKeys"] = \
+                    settings.DESKTOP_MODIFIER_KEYS
+            else:
+                self.prefsDict["keyboardLayout"] = \
+                    settings.GENERAL_KEYBOARD_LAYOUT_LAPTOP
+                self.prefsDict["orcaModifierKeys"] = \
+                    settings.LAPTOP_MODIFIER_KEYS
+
     def helpButtonClicked(self, widget):
         """Signal handler for the "clicked" signal for the helpButton
            GtkButton widget. The user has clicked the Help button.
@@ -1360,31 +1673,22 @@ class orcaSetupGUI(GladeWrapper):
 
         print "Help not currently implemented."
 
-    def cancelButtonClicked(self, widget):
-        """Signal handler for the "clicked" signal for the cancelButton
-           GtkButton widget. The user has clicked the Cancel button.
-           Don't write out the preferences. Hide the configuration window.
-
-        Arguments:
-        - widget: the component that generated the signal.
-        """
-
-        self.orcaSetupWindow.hide()
-
-    def okButtonClicked(self, widget):
-        """Signal handler for the "clicked" signal for the okButton
-           GtkButton widget. The user has clicked the OK button.
+    def applyButtonClicked(self, widget):
+        """Signal handler for the "clicked" signal for the applyButton
+           GtkButton widget. The user has clicked the Apply button.
            Write out the users preferences. If GNOME accessibility hadn't
            previously been enabled, warn the user that they will need to
            log out. Shut down any active speech servers that were started.
            Reload the users preferences to get the new speech, braille and
-           key echo value to take effect. Hide the configuration window.
+           key echo value to take effect. Do not dismiss the configuration
+           window.
 
         Arguments:
         - widget: the component that generated the signal.
         """
 
-        self.prefsDict["enableSpeech"] = True
+        enable = self.speechSupportCheckbutton.get_active()
+        self.prefsDict["enableSpeech"] = enable
         self.prefsDict["speechServerFactory"] = self.factory
         self.prefsDict["speechServerInfo"] = self.server
         self.prefsDict["voices"] = {
@@ -1400,6 +1704,33 @@ class orcaSetupGUI(GladeWrapper):
         for factory in self.workingFactories:
             factory.SpeechServer.shutdownActiveServers()
         orca.loadUserSettings()
+        self._populateKeyBindings()
+
+    def cancelButtonClicked(self, widget):
+        """Signal handler for the "clicked" signal for the cancelButton
+           GtkButton widget. The user has clicked the Cancel button.
+           Don't write out the preferences. Destroy the configuration window.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.orcaSetupWindow.destroy()
+
+    def okButtonClicked(self, widget):
+        """Signal handler for the "clicked" signal for the okButton
+           GtkButton widget. The user has clicked the OK button.
+           Write out the users preferences. If GNOME accessibility hadn't
+           previously been enabled, warn the user that they will need to
+           log out. Shut down any active speech servers that were started.
+           Reload the users preferences to get the new speech, braille and
+           key echo value to take effect. Hide the configuration window.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.applyButtonClicked(widget)
         self.orcaSetupWindow.hide()
 
     def windowDestroyed(self, widget):
