@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: Evolution.py 1248 2006-07-28 17:34:03Z wwalker $"
-__version__   = "$Revision: 1248 $"
-__date__      = "$Date: 2006-07-28 13:34:03 -0400 (Fri, 28 Jul 2006) $"
+__id__        = "$Id: Evolution.py,v 1.74 2006/11/20 19:21:57 richb Exp $"
+__version__   = "$Revision: 1.74 $"
+__date__      = "$Date: 2006/11/20 19:21:57 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -29,12 +29,14 @@ import orca.debug as debug
 import orca.default as default
 import orca.atspi as atspi
 import orca.input_event as input_event
+import orca.keybindings as keybindings
 import orca.rolenames as rolenames
 import orca.braille as braille
 import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.settings as settings
 import orca.util as util
+import orca.chnames as chnames
 
 from orca.orca_i18n import _ # for gettext support
 
@@ -53,12 +55,12 @@ class Script(default.Script):
         - app: the application to create a script for.
         """
 
-        default.Script.__init__(self, app)
-
         # Set the debug level for all the methods in this script.
         #
         self.debugLevel = debug.LEVEL_FINEST
 
+        default.Script.__init__(self, app)
+
         # This will be used to cache a handle to the message area in the
         # Mail compose window.
 
@@ -77,6 +79,10 @@ class Script(default.Script):
         self.lastMessageColumn = -1
         self.lastMessageRow = -1
 
+        # By default, don't present if Evolution is not the active application.
+        #
+        self.presentIfInactive = False
+
         # Evolution defines new custom roles. We need to make them known
         # to Orca for Speech and Braille output.
 
@@ -102,9 +108,52 @@ class Script(default.Script):
 
         default.Script.setupInputEventHandlers(self)
 
-        self.sayAllHandler = input_event.InputEventHandler(
-            Script.sayAll,
-            _("Speaks entire document."))
+        self.inputEventHandlers["sayAllHandler"] = \
+            input_event.InputEventHandler(
+                Script.sayAll,
+                _("Speaks entire document."))
+
+        self.inputEventHandlers["toggleReadMailHandler"] = \
+            input_event.InputEventHandler(
+                Script.toggleReadMail,
+                _("Toggle whether we present new mail if we not not the active script."))
+
+    def getKeyBindings(self):
+        """Defines the new key binding for this script. Setup the default
+        key bindings, then add one in for toggling whether we present new
+        mail if we not not the active script.
+
+        Returns an instance of keybindings.KeyBindings.
+        """
+
+        debug.println(self.debugLevel, "Evolution.getKeyBindings.")
+
+        keyBindings = default.Script.getKeyBindings(self)
+
+        keyBindings.add(
+            keybindings.KeyBinding(
+                "n",
+                1 << settings.MODIFIER_ORCA,
+                1 << settings.MODIFIER_ORCA,
+                self.inputEventHandlers["toggleReadMailHandler"]))
+        return keyBindings
+
+    def toggleReadMail(self, inputEvent):
+        """ Toggle whether we present new mail if we not not the active script.+
+        Arguments:
+        - inputEvent: if not None, the input event that caused this action.
+        """
+
+        debug.println(self.debugLevel, "Evolution.toggleReadMail.")
+
+        line = _("present new mail if this script is not active.")
+        self.presentIfInactive = not self.presentIfInactive
+        if not self.presentIfInactive:
+            line = _("do not present new mail if this script is not active.")
+
+        speech.speak(line)
+
+        return True
 
     def speakSetupAssistantLabel(self, label):
         """Perform a variety of tests on this Setup Assistant label to see
@@ -137,7 +186,8 @@ class Script(default.Script):
             if label.text:
                 charAttributes = label.text.getAttributes(0)
                 if charAttributes[0]:
-                    charDict = self.textAttrsToDictionary(charAttributes[0])
+                    [charKeys, charDict] = \
+                        self.textAttrsToDictionary(charAttributes[0])
                     weight = charDict.get('weight')
                     if weight and weight == '800':
                         text = util.getDisplayedText(label)
@@ -335,11 +385,18 @@ class Script(default.Script):
                           + "current message pane: " \
                           + "individual lines of text.")
 
-            result = util.getTextLineAtCaret(event.source)
-            braille.displayMessage(result[0])
+            [string, caretOffset, startOffset] = \
+                util.getTextLineAtCaret(event.source)
+            braille.displayMessage(string)
             if settings.enableSpeechIndentation:
-                self.speakTextIndentation(event.source, result[0])
-            speech.speak(result[0])
+                self.speakTextIndentation(event.source, string)
+            line = util.adjustForRepeats(string)
+                
+            if self.speakNewLine(event.source):
+                speech.speak(chnames.getCharacterName("\n"), None, False)
+ 
+            speech.speak(line, None, False)
+
             return
 
         # 2) Mail view: current message pane: "standard" mail header lines.
@@ -387,8 +444,9 @@ class Script(default.Script):
 
                     if cell.role == rolenames.ROLE_TEXT:
                         regions.append(braille.Text(cell))
-                        result = util.getTextLineAtCaret(cell)
-                        utterances.append(result[0])
+                        [string, caretOffset, startOffset] = \
+                            util.getTextLineAtCaret(cell)
+                        utterances.append(string)
 
                 braille.displayRegions([regions, regions[0]])
                 speech.speakUtterances(utterances)
@@ -495,6 +553,9 @@ class Script(default.Script):
                                 break
 
                     if toRead:
+                        obj = parent.table.getColumnHeader(i)
+                        header = atspi.Accessible.makeAccessible(obj)
+
                         # Speak/braille the column header for this table cell
                         # if it has focus (unless it's a checkbox).
                         if not checkbox and verbose:
@@ -503,8 +564,6 @@ class Script(default.Script):
                             settings.speechVerbosityLevel = \
                                 settings.VERBOSITY_LEVEL_BRIEF
 
-                            obj = parent.table.getColumnHeader(i)
-                            header = atspi.Accessible.makeAccessible(obj)
                             utterances = speechGen.getSpeech(header, False)
                             [headerRegions, focusedRegion] = \
                                          brailleGen.getBrailleRegions(header)
@@ -518,6 +577,11 @@ class Script(default.Script):
 
                         # Speak/braille the table cell.
                         #
+                        # If this cell has a column header of "Status",
+                        # then speak/braille "read".
+                        # If this cell has a column header of "Attachment",
+                        # then speak/braille "attachment".
+                        #
                         if verbose:
                             settings.brailleVerbosityLevel = \
                                 settings.VERBOSITY_LEVEL_VERBOSE
@@ -529,8 +593,20 @@ class Script(default.Script):
                         utterances = speechGen.getSpeech(cell, False)
                         [cellRegions, focusedRegion] = \
                                            brailleGen.getBrailleRegions(cell)
-                        brailleRegions.extend(cellRegions)
-                        brailleRegions.append(braille.Region(" "))
+
+                        if header.name == _("Status"):
+                            text = _("Read")
+                            utterances = [ text ]
+                            brailleRegions.append(braille.Region(text))
+                            brailleRegions.append(braille.Region(" "))
+                        elif header.name == _("Attachment"):
+                            text = header.name
+                            utterances = [ text ]
+                            brailleRegions.append(braille.Region(text))
+                            brailleRegions.append(braille.Region(" "))
+                        else:
+                            brailleRegions.extend(cellRegions)
+                            brailleRegions.append(braille.Region(" "))
 
                         # If the current focus is on a checkbox then we won't
                         # have set braille line focus to its header above, so
@@ -786,7 +862,11 @@ class Script(default.Script):
                           + "compose window: message area.")
 
             self.message_panel = event.source.parent.parent
+            
+            if self.speakNewLine(event.source):
+                speech.speak(chnames.getCharacterName("\n"), None, False)
 
+                
         # 9) Spell Checking Dialog
         #
         # This works in conjunction with code in section 8). Check to see if
@@ -842,7 +922,7 @@ class Script(default.Script):
                 panel = self.message_panel
                 allText = util.findByRole(panel, rolenames.ROLE_TEXT)
                 for i in range(0, len(allText)):
-                    text = allText[i].text.getText(0, -1)
+                    text = self.getText(allText[i], 0, -1)
                     tokens = text.split()
                     allTokens += tokens
 
@@ -883,7 +963,7 @@ class Script(default.Script):
             table = event.source.parent.parent.parent.parent.parent
             cell = table.child(table.childCount-1)
             allText = util.findByRole(cell, rolenames.ROLE_TEXT)
-            utterance = "for " + allText[0].text.getText(0, -1)
+            utterance = "for " + self.getText(allText[0], 0, -1)
             speech.speak(utterance)
             return
 
@@ -915,6 +995,62 @@ class Script(default.Script):
         default.Script.locusOfFocusChanged(self, event,
                                            oldLocusOfFocus, newLocusOfFocus)
 
+    def speakNewLine(self, obj):
+        """Returns True if a newline should be spoken.
+           Otherwise, returns False.
+        """
+
+        # Get the the AccessibleText interrface.
+        text = obj.text
+        if not text:
+            return False
+
+        # Was a left or right-arrow key pressed?
+        if not (orca_state.lastInputEvent and \
+                orca_state.lastInputEvent.__dict__.has_key("event_string")):
+            return False
+        
+        lastKey = orca_state.lastInputEvent.event_string
+        if lastKey != "Left" and lastKey != "Right":
+            return False
+
+        # Was a control key pressed?
+        mods = orca_state.lastInputEvent.modifiers
+        isControlKey = mods & (1 << atspi.Accessibility.MODIFIER_CONTROL)
+        
+        # Get the line containing the caret
+        caretOffset = text.caretOffset
+        line = text.getTextAtOffset(caretOffset, \
+            atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+        lineStart = line[1]
+        lineEnd = line[2]
+        
+        if isControlKey:  # control-right-arrow or control-left-arrow
+
+            # Get the word containing the caret.
+            word = text.getTextAtOffset(caretOffset, \
+                atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
+            wordStart = word[1]
+            wordEnd = word[2]
+            
+            if lastKey == "Right":
+                if wordStart == lineStart:
+                    return True
+            else: 
+                if wordEnd == lineEnd:
+                    return True
+                
+        else:  # right arrow or left arrow
+            
+            if lastKey == "Right":
+                if caretOffset == lineStart:
+                    return True
+            else: 
+                if caretOffset == lineEnd:
+                    return True
+
+        return False
+
     def onStateChanged(self, event):
         """Called whenever an object's state changes.  We are only
         interested in "object:state-changed:showing" events for any
