@@ -19,9 +19,9 @@
 
 """Custom script for Mozilla.  NOT WORKING WELL AT THE MOMENT."""
 
-__id__        = "$Id: Mozilla.py,v 1.14 2006/09/28 13:15:36 wwalker Exp $"
-__version__   = "$Revision: 1.14 $"
-__date__      = "$Date: 2006/09/28 13:15:36 $"
+__id__        = "$Id: Mozilla.py,v 1.15 2006/10/19 18:16:54 richb Exp $"
+__version__   = "$Revision: 1.15 $"
+__date__      = "$Date: 2006/10/19 18:16:54 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -30,3 +30,7 @@ import orca.Gecko as Gecko
 class Script(Gecko.Script):
     def __init__(self, app):
         Gecko.Script.__init__(self, app)
+
+        # By default, don't present if Mozilla is not the active application.
+        #
+        self.presentIfInactive = False
