@@ -19,19 +19,21 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: StarOffice.py,v 1.56 2006/10/14 15:17:52 wwalker Exp $"
-__version__   = "$Revision: 1.56 $"
-__date__      = "$Date: 2006/10/14 15:17:52 $"
+__id__        = "$Id: StarOffice.py,v 1.66 2006/12/01 00:58:58 richb Exp $"
+__version__   = "$Revision: 1.66 $"
+__date__      = "$Date: 2006/12/01 00:58:58 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import orca.debug as debug
 import orca.atspi as atspi
+import orca.chnames as chnames
 import orca.default as default
 import orca.input_event as input_event
 import orca.rolenames as rolenames
 import orca.braille as braille
 import orca.braillegenerator as braillegenerator
+import orca.orca as orca
 import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.speechgenerator as speechgenerator
@@ -184,8 +186,8 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         if isSpreadSheetCell(obj):
 
             # Check to see if this spread sheet cell has either a dynamic
-            # column heading or row heading (or both) associated with it. 
-            # If it does, then braille those first before brailling the 
+            # column heading or row heading (or both) associated with it.
+            # If it does, then braille those first before brailling the
             # cell contents.
             #
             table = getCalcTable(obj)
@@ -311,7 +313,9 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         return regions
 
 class SpeechGenerator(speechgenerator.SpeechGenerator):
-    """Overrides _getSpeechForTableCellRow so that , when we are in a
+    """Overrides _getSpeechForComboBox so that we can provide a name for
+    the Calc Name combo box.
+    Overrides _getSpeechForTableCellRow so that , when we are in a
     spread sheet, we can speak the dynamic row and column headers
     (assuming they are set).
     Overrides _getSpeechForTableCell so that, when we are in a spread
@@ -321,6 +325,45 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
     def __init__(self, script):
         speechgenerator.SpeechGenerator.__init__(self, script)
 
+    def _getSpeechForComboBox(self, obj, already_focused):
+        """Get the speech for a combo box. If the combo box already has focus,
+        then only the selection is spoken.
+        Also provides a name for the OOo Calc Name combo box. This name is
+        provided in clause 5) of locusOfFocusChanged() below.
+
+        Arguments:
+        - obj: the combo box
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = []
+
+        if not already_focused:
+            label = self._getSpeechForObjectLabel(obj)
+            if not label:
+                label = [ obj.name ]
+            utterances.extend(label)
+        else:
+            label = None
+
+        name = self._getSpeechForObjectName(obj)
+        if name != label:
+            utterances.extend(name)
+
+        if not already_focused:
+            utterances.extend(self._getSpeechForObjectRole(obj))
+
+        utterances.extend(self._getSpeechForObjectAvailability(obj))
+
+        self._debugGenerator("_getSpeechForComboBox",
+                             obj,
+                             already_focused,
+                             utterances)
+
+        return utterances
+
     def _getSpeechForTableCellRow(self, obj, already_focused):
         """Get the speech for a table cell row or a single table cell
         if settings.readTableCellRow is False. If this isn't inside a
@@ -427,21 +470,15 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
         if isSpreadSheetCell(obj):
             utterances = []
 
-            utterances.append(util.getDisplayedText(\
-                    util.getRealActiveDescendant(obj)))
-
             if inputLineForCell == None:
                 inputLineForCell = locateInputLine(obj)
 
-            # If the spread sheet table cell has something in it, then we
-            # want to append the name of the cell (which will be its location).
-            # Note that if the cell was empty, then util.getDisplayedText will
-            # have already done this for us.
-            #
             if obj.text:
                 objectText = self._script.getText(obj, 0, -1)
-                if objectText and len(objectText) != 0:
-                    utterances.append(" " + obj.name)
+                utterances.append(objectText)
+
+            nameList = obj.name.split()
+            utterances.append(nameList[1])
         else:
             speechGen = speechgenerator.SpeechGenerator
             utterances = speechGen._getSpeechForTableCell(self, obj,
@@ -488,6 +525,11 @@ class Script(default.Script):
 
         self.lastDynamicEvent = None
 
+        # Used to determine whether the caret has moved to a new paragraph.
+        #
+        self.currentParagraph = None
+
+
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
@@ -618,7 +660,7 @@ class Script(default.Script):
         return column
 
     def setDynamicColumnHeaders(self, inputEvent):
-        """Set the row for the dynamic header columns to use when speaking 
+        """Set the row for the dynamic header columns to use when speaking
         calc cell entries. In order to set the row, the user should first set
         focus to the row that they wish to define and then press Insert-r.
 
@@ -643,18 +685,46 @@ class Script(default.Script):
             if clickCount == 2:
                 try:
                     del dynamicColumnHeaders[table]
+                    line = _("Dynamic column header cleared.")
+                    speech.speak(line)
+                    braille.displayMessage(line)
                 except:
                     pass
             else:
                 dynamicColumnHeaders[table] = row
+                line = _("Dynamic column header set for row ") + str(row+1)
+                speech.speak(line)
+                braille.displayMessage(line)
         self.lastDynamicEvent = inputEvent
 
         return True
 
+    def columnConvert(self, column):
+        """ Convert a spreadsheet column into it's column label
+
+        Arguments:
+        - column: the column number to convert.
+
+        Returns a string representing the spread sheet column.
+        """
+
+        BASE26="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+
+        if column <= len(BASE26):
+            return BASE26[column-1]
+
+        res = ""
+        while column > 0:
+            digit = column % len(BASE26)
+            res = " " + BASE26[digit-1] + res
+            column /= len(BASE26)
+
+        return res
+
     def setDynamicRowHeaders(self, inputEvent):
-        """Set the column for the dynamic header rows to use when speaking 
-        calc cell entries. In order to set the column, the user should first 
-        set focus to the column that they wish to define and then press 
+        """Set the column for the dynamic header rows to use when speaking
+        calc cell entries. In order to set the column, the user should first
+        set focus to the column that they wish to define and then press
         Insert-c.
 
         Once the user has defined the column, it will be used to first speak
@@ -678,10 +748,18 @@ class Script(default.Script):
             if clickCount == 2:
                 try:
                     del dynamicRowHeaders[table]
+                    line = _("Dynamic row header cleared.")
+                    speech.speak(line)
+                    braille.displayMessage(line)
                 except:
                     pass
             else:
                 dynamicRowHeaders[table] = column
+                line = _("Dynamic row header set for column %s") % \
+                       self.columnConvert(column+1)
+                speech.speak(line)
+                braille.displayMessage(line)
+
         self.lastDynamicEvent = inputEvent
 
         return True
@@ -854,9 +932,12 @@ class Script(default.Script):
             self.speakSetupLabel(allLabels[i])
 
     # This method tries to detect and handle the following cases:
+    # 0) Writer: find command.
     # 1) Writer: text paragraph.
     # 2) Writer: spell checking dialog.
     # 3) Welcome to StarOffice dialog.
+    # 4) Calc: cell editor.
+    # 5) Calc: name box.
 
     def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
         """Called when the visual object with focus changes.
@@ -876,6 +957,17 @@ class Script(default.Script):
 
         # util.printAncestry(event.source)
 
+        # 0) Writer: find command
+        #
+        # Check to see if this is this is for the find command. See 
+        # comment #18 of bug #354463.
+        #
+        if self.findCommandRun and \
+           event.type == "object:state-changed:focused":
+            self.findCommandRun = False
+            self.find()
+            return
+
         # 1) Writer: text paragraph.
         #
         # We need to handle two things here:
@@ -885,8 +977,8 @@ class Script(default.Script):
         # paragraph, then we've just created the first new text document
         # in Writer. Announce it by doing a "where am I".
         #
-        # Also, when the focus is on a paragraph in the Document view of 
-        # the Writer, then just speak/braille the current line (rather than 
+        # Also, when the focus is on a paragraph in the Document view of
+        # the Writer, then just speak/braille the current line (rather than
         # speaking a bogus initial "paragraph" utterance as well).
 
         rolesList = [rolenames.ROLE_PARAGRAPH, \
@@ -1061,6 +1153,46 @@ class Script(default.Script):
                     + "Registration: Register Now radio button.")
                 self.handleSetupPanel(event.source.parent)
 
+        # 4) Calc: cell editor.
+        #
+        # Check to see if we are editing a spread sheet cell. If so, just
+        # return to avoid uttering something like "Paragraph 0 paragraph".
+        #
+        rolesList = [rolenames.ROLE_PARAGRAPH, \
+                     rolenames.ROLE_PANEL, \
+                     rolenames.ROLE_UNKNOWN, \
+                     rolenames.ROLE_SCROLL_PANE, \
+                     rolenames.ROLE_PANEL, \
+                     rolenames.ROLE_ROOT_PANE, \
+                     rolenames.ROLE_FRAME, \
+                     rolenames.ROLE_APPLICATION]
+        if util.isDesiredFocusedItem(event.source, rolesList):
+            debug.println(self.debugLevel, "StarOffice.locusOfFocusChanged - " \
+                          + "Calc: cell editor.")
+            return
+
+        # 5) Calc: name box
+        #
+        # Check to see if the focus has just moved to the Name Box combo
+        # box in Calc. If so, then replace the non-existent name with a
+        # simple one before falling through and calling the default 
+        # locusOfFocusChanged method, which in turn will result in our
+        # _getSpeechForComboBox() method being called.
+        #
+        rolesList = [rolenames.ROLE_COMBO_BOX, \
+                     rolenames.ROLE_TOOL_BAR, \
+                     rolenames.ROLE_PANEL, \
+                     rolenames.ROLE_ROOT_PANE, \
+                     rolenames.ROLE_FRAME, \
+                     rolenames.ROLE_APPLICATION]
+
+        if util.isDesiredFocusedItem(event.source, rolesList) \
+            and (not event.source.name or len(event.source.name) == 0):
+            debug.println(self.debugLevel, "StarOffice.locusOfFocusChanged - " \
+                          + "Calc: name box.")
+
+            event.source.name = _("Move to cell")
+
         # Pass the event onto the parent class to be handled in the default way.
 
         default.Script.locusOfFocusChanged(self, event,
@@ -1157,6 +1289,63 @@ class Script(default.Script):
 
         default.Script.onNameChanged(self, event)
 
+
+    def onFocus(self, event):
+        """Called whenever an object gets focus. Overridden in this script 
+        so that we can adjust "focus:" events for children of a combo-box 
+        to just set the focus to the combo box. This is needed to help 
+        reduce the verbosity of focusing on the Calc Name combo box (see 
+        bug #364407).
+
+        Arguments:
+        - event: the Event
+        """
+
+        if event.source.parent.role == rolenames.ROLE_COMBO_BOX:
+            orca.setLocusOfFocus(None, event.source.parent, False)
+            return
+
+        default.Script.onFocus(self, event)
+
+    def onStateChanged(self, event):
+        """Called whenever an object's state changes.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # Prevent  "object:state-changed:active" events from activating
+        # the find operation. See comment #18 of bug #354463.
+        #
+        if event.type == "object:state-changed:active":
+            if self.findCommandRun:
+                return
+
+        # Two events are received when the caret moves
+        # to a new paragraph. The first is a focus event
+        # (in the form of object:state-changed:focused
+        # instead of focus:). The second is a caret-moved
+        # event. Just set the locusOfFocus for the first event.
+        # 
+        if event.type == "object:state-changed:focused" and \
+           event.source.role == rolenames.ROLE_PARAGRAPH and \
+           event.source != self.currentParagraph:
+            self.currentParagraph = event.source
+            orca.setLocusOfFocus(event, event.source, False)
+            return
+
+        # If we get "object:state-changed:focused" events for children of 
+        # a combo-box, just set the focus to the combo box. This is needed 
+        # to help reduce the verbosity of focusing on the Calc Name combo 
+        # box (see bug #364407).
+        #
+        if event.source.parent.role == rolenames.ROLE_COMBO_BOX:
+            orca.setLocusOfFocus(None, event.source.parent, False)
+            return
+
+        default.Script.onStateChanged(self, event)
+
+
     # This method tries to detect and handle the following cases:
     # 1) Calc: spread sheet Name Box line.
 
@@ -1247,3 +1436,99 @@ class Script(default.Script):
         string = text[max(0,startOffset):min(len(text),endOffset)]
         string = string.encode("UTF-8")
         return string
+
+    def onCaretMoved(self, event):
+        """Called whenever the caret moves.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # Speak a newline, if appropriate.
+        if self.speakNewLine(event.source):
+            speech.speak(chnames.getCharacterName("\n"), None, False)
+
+        # Speak a blank line, if appropriate.
+        if self.speakBlankLine(event.source):
+            speech.speak(_("blank"), None, False)
+
+        default.Script.onCaretMoved(self, event)
+
+
+    def speakNewLine(self, obj):
+        """Returns True if a newline should be spoken.
+           Otherwise, returns False.
+        """
+
+        # Get the the AccessibleText interrface.
+        text = obj.text
+        if not text:
+            return False
+
+        # Was a left or right-arrow key pressed?
+        if not (orca_state.lastInputEvent and \
+                orca_state.lastInputEvent.__dict__.has_key("event_string")):
+            return False
+        
+        lastKey = orca_state.lastInputEvent.event_string
+        if lastKey != "Left" and lastKey != "Right":
+            return False
+
+        # Was a control key pressed?
+        mods = orca_state.lastInputEvent.modifiers
+        isControlKey = mods & (1 << atspi.Accessibility.MODIFIER_CONTROL)
+        
+        # Get the line containing the caret
+        caretOffset = text.caretOffset
+        line = text.getTextAtOffset(caretOffset, \
+            atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+        lineStart = line[1]
+        lineEnd = line[2]
+
+        if isControlKey:  # control-right-arrow or control-left-arrow
+
+            # Get the word containing the caret.
+            word = text.getTextAtOffset(caretOffset, \
+                atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
+            wordStart = word[1]
+            wordEnd = word[2]
+            
+            if lastKey == "Right":
+                if wordStart == lineStart:
+                    return True
+            else: 
+                if wordEnd == lineEnd:
+                    return True
+                
+        else:  # right arrow or left arrow
+            
+            if lastKey == "Right":
+                if caretOffset == lineStart:
+                    return True
+            else: 
+                if caretOffset == lineEnd:
+                    return True
+
+        return False
+
+
+    def speakBlankLine(self, obj):
+        """Returns True if a blank line should be spoken.
+        Otherwise, returns False.
+        """
+        
+        # Get the the AccessibleText interrface. 
+        text = obj.text
+        if not text:
+            return False
+
+        # Get the line containing the caret
+        caretOffset = text.caretOffset
+        line = text.getTextAtOffset(caretOffset, \
+            atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+
+        # If this is a blank line, announce it if the user requested
+        # that blank lines be spoken.
+        if line[1] == 0 and line[2] == 0:
+            return settings.speakBlankLines
+
