@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py,v 1.84 2006/11/02 16:07:46 richb Exp $"
-__version__   = "$Revision: 1.84 $"
-__date__      = "$Date: 2006/11/02 16:07:46 $"
+__id__        = "$Id: settings.py,v 1.91 2006/11/30 16:38:58 richb Exp $"
+__version__   = "$Revision: 1.91 $"
+__date__      = "$Date: 2006/11/30 16:38:58 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -92,6 +92,8 @@ userCustomizableSettings = [
     "magTargetDisplay",
     "verbalizePunctuationStyle",
     "showMainWindow",
+    "keyboardLayout",
+    "speakBlankLines",   
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -113,6 +115,11 @@ consolePreferencesModule= "orca_console_prefs"
 #
 quitModule = "orca_quit"
 
+# The name of the module that holds the user interface for performing a
+# flat review find.
+#
+findModule = "orca_gui_find"
+
 # A list of keys that can serve as the Orca modifier key.  The list is
 # so we can provide better cross platform support (e.g., Sun keyboard
 # vs. PC-104 keyboard layouts).  When any of these keys is pressed,
@@ -120,7 +127,12 @@ quitModule = "orca_quit"
 # a KeyboardEvent input event.  The keys are currently compared to the
 # event_string of a keyboard input event from AT-SPI.
 #
-orcaModifierKeys        = ["Insert", "KP_Insert"]
+# The initial set of modifier keys is dependant upon whether the user
+# has specified a desktop or a laptop keyboard layout.
+#
+DESKTOP_MODIFIER_KEYS = ["Insert", "KP_Insert"]
+LAPTOP_MODIFIER_KEYS  = ["Caps_Lock"]
+orcaModifierKeys      = DESKTOP_MODIFIER_KEYS
 
 # A new modifier to use (set by the press of any key in the
 # orcaModifierKeys list) to represent the Orca modifier.
@@ -188,7 +200,7 @@ HYPERLINK_VOICE         = "hyperlink"
 voices = {
     DEFAULT_VOICE   : ACSS({}),
     UPPERCASE_VOICE : ACSS({ACSS.AVERAGE_PITCH : 6}),
-    HYPERLINK_VOICE : ACSS({ACSS.AVERAGE_PITCH : 8})
+    HYPERLINK_VOICE : ACSS({})
 }
 
 # If True, enable speaking of speech indentation and justification.
@@ -314,12 +326,31 @@ enableActionKeys        = True
 #
 showMainWindow          = True
 
+# Keyboard layout options (see keyboardLayout).
+#
+GENERAL_KEYBOARD_LAYOUT_DESKTOP = 1
+GENERAL_KEYBOARD_LAYOUT_LAPTOP  = 2
+keyboardLayout                  = GENERAL_KEYBOARD_LAYOUT_DESKTOP
+
+# If True, speak blank lines.
+#
+speakBlankLines         = True
 
 # If True, reads all the table cells in the current row rather than just
 # the current one.
 #
 readTableCellRow    = True
 
+# The default set of text attributes to speak to the user. Specific
+# application scripts (or individual users can override these values if
+# so desired. Each of these text attributes is of the form <key>:<value>;
+# The <value> part will be the "default" value for that attribute. In
+# other words, if the attribute for a given piece of text has that value,
+# it won't be spoken. If no value part is given, then that attribute will
+# always be spoken.
+
+enabledTextAttributes = "size:; family-name:; weight:400; indent:0; underline:none; strikethrough:false; justification:left; style:normal;"
+
 # The limit to enable a repeat character count to be spoken.
 # If set to 0, then there will be no repeat character count.
 # Each character will be spoken singularly (i.e. "dash dash
@@ -415,6 +446,14 @@ commFailureAttemptLimit = 5
 #
 doubleClickTimeout = 0.5
 
+# Which packages to search, and the order in which to search,
+# for application settings.  These packages are expected to be on
+# the PYTHONPATH and/or subpackages of the "orca" package.
+# REMEMBER: to make something a package, the directory has to
+# have a __init__.py file in it.
+#
+settingsPackages          = ["app-settings"]
+
 # Which packages to search, and the order in which to search,
 # for custom scripts.  These packages are expected to be on
 # the PYTHONPATH and/or subpackages of the "orca" package.
@@ -487,3 +526,4 @@ setScriptMapping(re.compile(_('Bon Echo')), "Mozilla")
 setScriptMapping(re.compile(_('Minefield')), "Mozilla")
 setScriptMapping(re.compile(_('bug-buddy')), "gnome_segv2")
 setScriptMapping(re.compile(_('vte')), "gnome-terminal")
+setScriptMapping(re.compile(_('time-admin')), "users-admin")
