@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py,v 1.83 2006/10/25 00:31:48 wwalker Exp $"
-__version__   = "$Revision: 1.83 $"
-__date__      = "$Date: 2006/10/25 00:31:48 $"
+__id__        = "$Id: speechgenerator.py,v 1.87 2006/12/03 11:24:21 wwalker Exp $"
+__version__   = "$Revision: 1.87 $"
+__date__      = "$Date: 2006/12/03 11:24:21 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -52,7 +52,7 @@ class SpeechGenerator:
         # script for information if we need it.
         #
         self._script = script
-        
+
         # Set up a dictionary that maps role names to functions
         # that generate speech for objects that implement that role.
         #
@@ -91,6 +91,8 @@ class SpeechGenerator:
              self._getSpeechForImage
         self.speechGenerators[rolenames.ROLE_LABEL]               = \
              self._getSpeechForLabel
+        self.speechGenerators[rolenames.ROLE_LAYERED_PANE]        = \
+             self._getSpeechForLayeredPane
         self.speechGenerators[rolenames.ROLE_LIST]                = \
              self._getSpeechForList
         self.speechGenerators[rolenames.ROLE_MENU]                = \
@@ -672,6 +674,36 @@ class SpeechGenerator:
 
         return utterances
 
+    def _getSpeechForLayeredPane(self, obj, already_focused):
+        """Get the speech for a layered pane
+
+        Arguments:
+        - obj: the table
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = self._getDefaultSpeech(obj, already_focused)
+
+        self._debugGenerator("_getSpeechForLayeredPane",
+                             obj,
+                             already_focused,
+                             utterances)
+
+        # If this has no children, then let the user know.
+        #
+        hasItems = False
+        for i in range(0, obj.childCount):
+            child = obj.child(i)
+            if child.state.count(atspi.Accessibility.STATE_SHOWING):
+                hasItems = True
+                break
+        if not hasItems:
+            utterances.append(_("0 items"))
+
+        return utterances
+
     def _getSpeechForList(self, obj, already_focused):
         """Get the speech for a list.
 
@@ -782,7 +814,7 @@ class SpeechGenerator:
                 utterances.append(obj.name)
         utterances.extend(self._getSpeechForObjectRole(obj))
 
-        [text, startOffset, endOffset] = util.getTextLineAtCaret(obj)
+        [text, caretOffset, startOffset] = util.getTextLineAtCaret(obj)
         utterances.append(text)
 
         self._debugGenerator("_getSpeechForText",
@@ -1033,6 +1065,9 @@ class SpeechGenerator:
         try:
             minimumIncrement = value.minimumIncrement
         except:
+            pass
+
+        if minimumIncrement == 0.0:
             minimumIncrement = (value.maximumValue - value.minimumValue) \
                                / 100.0
 
@@ -1141,6 +1176,17 @@ class SpeechGenerator:
                              already_focused,
                              utterances)
 
+        # If this is a table with no children, then let the user know.
+        #
+        hasItems = False
+        for i in range(0, obj.childCount):
+            child = obj.child(i)
+            if child.state.count(atspi.Accessibility.STATE_SHOWING):
+                hasItems = True
+                break
+        if not hasItems:
+            utterances.append(_("0 items"))
+
         return utterances
 
     def _getSpeechForTableCell(self, obj, already_focused):
@@ -1185,6 +1231,12 @@ class SpeechGenerator:
             else:
                 utterances.append(_("collapsed"))
 
+            # If this is an expandable table cell with no children, then
+            # let the user know.
+            #
+            if obj.childCount == 0:
+                utterances.append(_("0 items"))
+
         self._debugGenerator("_getSpeechForTableCell",
                              obj,
                              already_focused,
@@ -1230,13 +1282,15 @@ class SpeechGenerator:
                         showing = cell.state.count( \
                                           atspi.Accessibility.STATE_SHOWING)
                         if showing:
-                            utterances.extend(self._getSpeechForTableCell(cell, 
+                            utterances.extend(self._getSpeechForTableCell(cell,
                                                            already_focused))
                 else:
                     utterances.extend(self._getSpeechForTableCell(obj,
                                                            already_focused))
             else:
                 utterances = self._getSpeechForTableCell(obj, already_focused)
+        else:
+            utterances = self._getSpeechForTableCell(obj, already_focused)
 
         return utterances
 
@@ -1342,7 +1396,7 @@ class SpeechGenerator:
             checkedState = _("pressed")
         else:
             checkedState = _("not pressed")
-            
+
         # If it's not already focused, say it's name
         #
         if not already_focused:
